#------------------------------------------------------------------
q.rot <- function(n,mu,sigma,weight=1,alpha=0.10,cutoff=0) {
# Xplain: Computes the "informed rule of thumb" for Bugni & Canay (2018)
# Last modified : June 17, 2019 by Ivan Canay
  #------------------------------------------------------------------
  # INPUTS: 
  # 	n : sample size
  # 	mu : sample mean of the running variable
  #   weight : the weight multiplies the preliminary rot as a robustness check 
  # 	sigma  : sample sd of the running variable 
  # 	alpha : significance level 
  # 	cutoff : cutoff point (threshold) 
  #------------------------------------------------------------------
  # RETURNS:
  # 	- q.rot: the informed rule of thumb
  # 	- q.prelim: the preliminary rule of thumb 
  # 	- limit.prob : the limit probability (under the null) of the non-randomized test
  #------------------------------------------------------------------
  # WARNINGS and STOPS
  if (length(n)>1){stop("Sample size must be a scalar")}
  if (length(mu)>1){stop("mu must be a scalar")}
  if (length(sigma)>1){stop("mu must be a scalar")}
  if (n<20){stop("Sample size too small")}
  if (sigma<=0){stop("sigma must be positive")}
  #------------------------------------------------------------------
  # Step 1: Compute the preliminary rule of thumb.
  
  lb = ceiling(1-log(alpha)/log(2));
  x  = (cutoff-mu)/sigma;
  constant = (sigma*4*dnorm(cutoff,mu,sigma)^2*(1/dnorm(mu+sigma,mu,sigma)))^(2/3);
  new.rule = weight*n^(1/2)*constant; 
  q.prelim = ceiling(max(lb,new.rule));
  
  max.points = ceiling(4*log(q.prelim));
  q_l = max(lb,q.prelim - max.points);
  q_h = q.prelim + max.points;		
  
  
  num.grid = q_h-q_l+1;
  grid = seq(q_l,q_h,1);
  nr.power = matrix(0,num.grid,1);
  
  for (j in 1:num.grid){
    q = grid[j];
    b=round(uniroot(function(x) 2*pbinom(x,q,0.5)-alpha,lower=0,upper=q/2)$root);
    nr.power[j] = 2*pbinom(b-1,q,0.5);
  }
  
  # Step 3: Compute the informed rule of thumb
  q.rot = grid[which.max(nr.power)];
  return(list(q.rot=q.rot,q.prelim=q.prelim,limit.prob=nr.power[which.max(nr.power)]))
}
#------------------------------------------------------------------